﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Text.RegularExpressions;
using System.IO;

namespace AdministrativeDivision
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
			InitializeScrollTextWorker();
			InitializePickNodeWorker();
			InitializeComboExport();
		}

		private BackgroundWorker _workerScrollText = new BackgroundWorker() { WorkerSupportsCancellation = true, WorkerReportsProgress = true };
		private BackgroundWorker _workerPickNode = new BackgroundWorker() { WorkerSupportsCancellation = true, WorkerReportsProgress = true };
		private string _stringScrollText = @"老毕出品，版权所有。http://www.cnblogs.com/abbey";
		private string _pathSourceFile = string.Empty;
		private string _pathExportFile = string.Empty;
		private const string PATTERN = @"^\s*(\d{6})\s+(\w+)\s*$";

		private List<ExportHandlerItem> _handlerList = new List<ExportHandlerItem>();
		private List<CityNode> _cityList = new List<CityNode>();
		private class ExportHandlerItem
		{
			public delegate void ExportHandler(string filepath);
			public string Name { get; set; }
			public string Filter { get; set; }
			public ExportHandler Handler { get; set; }
		}
		private void InitializeScrollTextWorker()
		{
			_workerScrollText.DoWork += new DoWorkEventHandler(
							(s, t) =>
							{
								int LENGTHOFSCROLLTEXT = _stringScrollText.Length;
								int shift = 0;

								while (true)
								{
									if (_workerScrollText.CancellationPending)
									{
										t.Cancel = true;
										return;
									}

									Thread.Sleep(500);
									_workerScrollText.ReportProgress(shift);

									shift++;
									if (shift > LENGTHOFSCROLLTEXT)
										shift = 0;
								}
							});

			_workerScrollText.ProgressChanged += new ProgressChangedEventHandler(
				(s, t) =>
				{
					StringBuilder temp = new StringBuilder();
					temp.Append(_stringScrollText.Substring(t.ProgressPercentage));
					temp.Append("    ");
					temp.Append(_stringScrollText.Substring(0, t.ProgressPercentage));

					this.labelCopyright.Text = temp.ToString();
				});
		}

		private void InitializePickNodeWorker()
		{
			_workerPickNode.DoWork += new DoWorkEventHandler(
					(s, t) =>
					{
						ListBox list = t.Argument as ListBox;
						if (list == null)
							return;
						else
						{
							_cityList.Clear();
							Regex regex = new Regex(PATTERN);
							Match m;

							int count = list.Items.Count;
							for (int i = 0; i < count; i++)
							{
								_workerPickNode.ReportProgress((int) ((float) i / (float) count * 100));
								if (_workerPickNode.CancellationPending)
								{
									t.Cancel = true;
									return;
								}

								m = regex.Match(list.Items[i].ToString());
								if (m.Success)
								{
									CityNode node = new CityNode() { Code = m.Groups[1].Value, Name = m.Groups[2].Value };

									if (node.Code.Substring(4).Equals("00"))
									{
										CityNode province = _cityList.Where(c => c.Code.Equals(node.Code.Substring(0, 2) + "0000")).SingleOrDefault();
										if (province != null)
											node.Parent = province;
									}
									else
									{
										CityNode prefectural = _cityList.Where(c => c.Code.Equals(node.Code.Substring(0, 4) + "00")).SingleOrDefault();
										if (prefectural != null)
											node.Parent = prefectural;
									}

									_cityList.Add(node);
								}
							}
						}
					});

			_workerPickNode.ProgressChanged += new ProgressChangedEventHandler(
				(s, t) =>
				{
					this.progressBar.Value = t.ProgressPercentage;
				});

			_workerPickNode.RunWorkerCompleted += new RunWorkerCompletedEventHandler(
				(s, t) =>
				{
					if (t.Cancelled)
					{
						_cityList.Clear();
					}

					EnableControls();
					this.progressBar.Visible = false;
					this.btnCancel.Visible = false;
				});
		}

		private void InitializeComboExport()
		{
			// initialization of comboExport.
			_handlerList.Add(new ExportHandlerItem { Name = "sql脚本", Filter = @"sql script (*.sql)|*.sql", Handler = ExportToSqlScript });
			_handlerList.Add(new ExportHandlerItem { Name = "Excel文件", Filter = @"Excel files (*.xls)|*.xls", Handler = ExportToExcel });
			_handlerList.Add(new ExportHandlerItem { Name = "Access文件", Filter = @"Access files (*.mdb)|*.mdb", Handler = ExportToAccess });

			this.comboExport.DataSource = _handlerList;
			this.comboExport.DisplayMember = "Name";
			this.comboExport.ValueMember = "Handler";
			this.comboExport.SelectedIndex = 0;
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			// start thread to scroll text of labelCopyright.
			this.labelCopyright.Text = _stringScrollText;
			_workerScrollText.RunWorkerAsync();
		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			_workerScrollText.CancelAsync();
		}

		private void webLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			System.Diagnostics.Process.Start(Properties.Settings.Default.WebURL);
		}

		private void btnOpenFile_Click(object sender, EventArgs e)
		{
			OpenFileDialog filedlg = new OpenFileDialog();
			filedlg.Filter = @"txt files (*.txt)|*.txt";
			filedlg.FilterIndex = 1;

			if (filedlg.ShowDialog() == DialogResult.OK)
			{
				this.treeDivision.Nodes.Clear();

				this._pathSourceFile = filedlg.FileName;
				this.txtFilePath.Text = this._pathSourceFile;
				LoadLinesFromFile();

				DisableControls();
				this.progressBar.Visible = true;
				this.btnCancel.Visible = true;

				_workerPickNode.RunWorkerAsync(this.listText);
				Point p = new Point(progressBar.Size.Width / 3, 0 - progressBar.Size.Height - 10);
				this.tipMain.Show("后台正在提取文本构造级联关系...", this.progressBar, p, 1000);
			}
		}

		private void btnParse_Click(object sender, EventArgs e)
		{
			DisableControls();
			BuildTree();
			EnableControls();
		}

		private void btnExport_Click(object sender, EventArgs e)
		{
			ExportHandlerItem.ExportHandler handler = (ExportHandlerItem.ExportHandler) this.comboExport.SelectedValue;

			if (handler != null)
			{
				SaveFileDialog filedlg = new SaveFileDialog();
				filedlg.Filter = _handlerList[this.comboExport.SelectedIndex].Filter;
				filedlg.FilterIndex = this.comboExport.SelectedIndex + 1;

				if (filedlg.ShowDialog() == DialogResult.OK)
				{
					_pathExportFile = filedlg.FileName;
					try
					{
						handler(this._pathExportFile);
						string message = string.Format("数据已成功导出至文件：\r\n  {0}", this._pathExportFile);
						MessageBox.Show(message, "导出成功", MessageBoxButtons.OK, MessageBoxIcon.Information);
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.Message, "导出失败", MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}
		}
		private void btnCancel_Click(object sender, EventArgs e)
		{
			_workerPickNode.CancelAsync();
		}

		private void ExportToSqlScript(string filepath)
		{
			const string header = @"USE [master]
	GO

	IF  EXISTS (SELECT name FROM sys.databases WHERE name = N'AdministrativeDivision')
	DROP DATABASE [AdministrativeDivision]
	GO

	USE [master]
	GO

	CREATE DATABASE [AdministrativeDivision]
	GO

	USE [AdministrativeDivision]
	GO

	IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Divisons]') AND type in (N'U'))
	DROP TABLE [dbo].[Divisons]
	GO

	CREATE TABLE [dbo].[Divisions](
	[ID] [nchar](6) NOT NULL,
	[Name] [nvarchar](50) NOT NULL,
	[Parent] [nchar](6) NULL,
	CONSTRAINT [PK_Divisons] PRIMARY KEY CLUSTERED 
	(
	[ID] ASC
	)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
	) ON [PRIMARY]
	GO";

			const string insertion = @"INSERT INTO [dbo].[Divisions] ([ID], [Name], [Parent]) VALUES ('{0}', '{1}', '{2}')";

			using (StreamWriter writer = new StreamWriter(filepath, false, UTF8Encoding.Default))
			{
				writer.WriteLine(header);
				foreach (CityNode c in _cityList)
				{
					writer.WriteLine(insertion, c.Code, c.Name, c.Parent == null ? string.Empty : c.Parent.Code);
				}
				writer.Close();
			}
		}

		private void ExportToExcel(string filepath)
		{
			throw new ApplicationException("Not implemented.");
		}

		private void ExportToAccess(string filepath)
		{
			throw new ApplicationException("Not implemented.");
		}

		private void LoadLinesFromFile()
		{
			using (System.IO.StreamReader reader = new System.IO.StreamReader(_pathSourceFile, true))
			{
				this.listText.Items.Clear();

				this.tipMain.SetToolTip(this.txtFilePath, string.Format("当前编码格式：{0}", reader.CurrentEncoding.ToString()));
				string line;
				try
				{
					while (!reader.EndOfStream)
					{
						line = reader.ReadLine();
						this.listText.Items.Add(line);
					}
				}
				catch (System.Exception ex)
				{
					MessageBox.Show(ex.Message, "读取文件失败", MessageBoxButtons.OK, MessageBoxIcon.Error);
					this.tipMain.SetToolTip(this.txtFilePath, string.Empty);
				}
			}
		}

		private void BuildTree()
		{
			this.treeDivision.Nodes.Clear();
			TreeNode root = new TreeNode("全国行政区划目录");
			this.treeDivision.Nodes.Add(root);

			Dictionary<string, TreeNode> cityByCode = new Dictionary<string, TreeNode>();
			TreeNode[] nodes = new TreeNode[_cityList.Count];

			int i = 0;
			foreach (CityNode city in _cityList.OrderBy(c => c.Code))
			{
				nodes[i] = new TreeNode();
				nodes[i].Text = city.ToString();
				nodes[i].Tag = city;
				cityByCode.Add(city.Code, nodes[i]);

				TreeNode parent;
				if (city.Parent != null)
				{
					if (cityByCode.TryGetValue(city.Parent.Code, out parent))
					{
						parent.Nodes.Add(nodes[i]);
					}
				}
				else
				{
					root.Nodes.Add(nodes[i]);
				}

				i++;
			}
			root.Expand();
		}

		private void EnableControls()
		{
			this.txtFilePath.Enabled = true;
			this.btnOpenFile.Enabled = true;
			this.listText.Enabled = true;
			this.comboExport.Enabled = true;
			this.btnExport.Enabled = true;
			this.btnParse.Enabled = true;
		}

		private void DisableControls()
		{
			this.txtFilePath.Enabled = false;
			this.btnOpenFile.Enabled = false;
			this.listText.Enabled = false;
			this.comboExport.Enabled = false;
			this.btnExport.Enabled = false;
			this.btnParse.Enabled = false;
		}

		private void btnTestDlg_Click(object sender, EventArgs e)
		{
			QueryDlg dlg = new QueryDlg();
			dlg.Cities = this._cityList;
			if (dlg.ShowDialog() == DialogResult.OK)
			{
				CityNode c = dlg.SelectedCity;
				string message = string.Format("{2}-{1}-{0} 被选择", c.Name, c.Parent.Name, c.Parent.Parent.Name);
				MessageBox.Show(message);
			}
		}

	}

	public class CityNode
	{
		public string Code { get; set; }
		public string Name { get; set; }
		public CityNode Parent { get; set; }

		public override string ToString()
		{
			if ((!string.IsNullOrEmpty(this.Code))
				&& (!string.IsNullOrEmpty(this.Name)))
				return string.Format("{0}-{1}", this.Name, this.Code);
			else
				return string.Empty;
		}

		public override bool Equals(object obj)
		{
			CityNode right = obj as CityNode;
			if (right == null)
				return false;
			else
				return this.Code.Equals(right.Code) && this.Name.Equals(right.Name);
		}
	}
}
